@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat, Inc.
+ * Copyright 2008,2011,2012,2013,2014 Red Hat, Inc.
  *
  * This Program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@
 #endif
 
 #include <rpc/xdr.h>
-#include <rpcsvc/yp_prot.h>
+#include "../yp/yp.h"
 
 #ifdef HAVE_TCPD_H
 #include <tcpd.h>
@@ -72,9 +72,10 @@ backend_free_master_name(struct plugin_state *state, char *master)
 	backend_shr_free_server_name(state, master);
 }
 int
-backend_read_master_name(struct plugin_state *state, char **master)
+backend_read_master_name(struct plugin_state *state, Slapi_PBlock *pb,
+			 char **master)
 {
-	return backend_shr_read_server_name(state, master);
+	return backend_shr_read_server_name(pb, state, master);
 }
 
 /* Manipulate map configuration data. */
@@ -86,6 +87,10 @@ backend_free_set_data_contents(void *data)
 		free(set_data->common.group);
 		free(set_data->common.set);
 		backend_shr_free_strlist(set_data->common.bases);
+		backend_shr_free_sdnlist(set_data->common.restrict_subtrees);
+		backend_shr_free_sdnlist(set_data->common.ignore_subtrees);
+		format_free_attr_list(set_data->common.rel_attrs);
+		free(set_data->common.rel_attr_list);
 		format_free_attr_list(set_data->common.ref_attrs);
 		format_free_inref_attrs(set_data->common.inref_attrs);
 		format_free_ref_attr_list(set_data->common.ref_attr_list);
@@ -108,6 +113,7 @@ static struct backend_shr_set_data *
 backend_copy_set_data(const struct backend_set_data *data)
 {
 	struct backend_set_data *ret;
+
 	ret = malloc(sizeof(*ret));
 	if (ret == NULL) {
 		return NULL;
@@ -117,7 +123,16 @@ backend_copy_set_data(const struct backend_set_data *data)
 	ret->common.group = strdup(data->common.group);
 	ret->common.set = strdup(data->common.set);
 	ret->common.bases = backend_shr_dup_strlist(data->common.bases);
-	ret->common.entry_filter = strdup(data->common.entry_filter);
+	ret->common.entry_filter = data->common.entry_filter ?
+				   strdup(data->common.entry_filter) :
+				   NULL;
+	ret->common.restrict_subtrees = backend_shr_dup_sdnlist(data->common.restrict_subtrees);
+	ret->common.ignore_subtrees = backend_shr_dup_sdnlist(data->common.ignore_subtrees);
+	ret->common.rel_attrs = data->common.rel_attrs ?
+				format_dup_attr_list(data->common.rel_attrs) :
+				NULL;
+	ret->common.rel_attr_list = NULL;
+	ret->common.rel_attrs_list = NULL;
 	ret->common.ref_attrs = data->common.ref_attrs ?
 				format_dup_attr_list(data->common.ref_attrs) :
 				NULL;
@@ -130,6 +145,8 @@ backend_copy_set_data(const struct backend_set_data *data)
 	ret->common.inref_attr_list = data->common.inref_attr_list ?
 				      format_dup_ref_attr_list(data->common.inref_attr_list) :
 				    NULL;
+	ret->common.skip_uninteresting_updates =
+		data->common.skip_uninteresting_updates;
 	ret->disallowed_chars = data->disallowed_chars ?
 				strdup(data->disallowed_chars) : NULL;
 	ret->key_formats = backend_shr_dup_strlist(data->key_formats);
@@ -180,10 +197,14 @@ backend_free_gathered_data(char **all, unsigned int *all_lengths,
 	free(group_lengths);
 }
 static char **
-backend_gather_data(struct plugin_state *state, Slapi_Entry *e,
+backend_gather_data(struct plugin_state *state,
+		    Slapi_PBlock *pb, Slapi_Entry *e,
 		    const char *domain, const char *map,
 		    char **single_formats, char **group_formats,
 		    const char *disallowed_chars,
+		    const struct slapi_dn **restrict_subtrees,
+		    const struct slapi_dn **ignore_subtrees,
+		    char ***rel_attrs,
 		    char ***ref_attrs,
 		    struct format_inref_attr ***inref_attrs,
 		    struct format_ref_attr_list ***ref_attr_list,
@@ -240,10 +261,12 @@ backend_gather_data(struct plugin_state *state, Slapi_Entry *e,
 	}
 	n = 0;
 	for (i = 0; i < n_singles; i++) {
-		singles[i] = format_get_data(state, e, domain, map,
+		singles[i] = format_get_data(state, pb, e, domain, map,
 					     single_formats[i],
 					     disallowed_chars,
-					     ref_attrs, inref_attrs,
+					     restrict_subtrees,
+					     ignore_subtrees,
+					     rel_attrs, ref_attrs, inref_attrs,
 					     ref_attr_list, inref_attr_list,
 					     &single_lengths[i]);
 		if (singles[i] != NULL) {
@@ -268,10 +291,12 @@ backend_gather_data(struct plugin_state *state, Slapi_Entry *e,
 		}
 	}
 	for (i = 0, j = 0; i < n_groups; i++) {
-		groups[j] = format_get_data_set(state, e, domain, map,
+		groups[j] = format_get_data_set(state, pb, e, domain, map,
 						group_formats[i],
 						disallowed_chars,
-						ref_attrs, inref_attrs,
+						restrict_subtrees,
+						ignore_subtrees,
+						rel_attrs, ref_attrs, inref_attrs,
 						ref_attr_list, inref_attr_list,
 						&group_lengths[j]);
 		if (groups[j] != NULL) {
@@ -283,10 +308,11 @@ backend_gather_data(struct plugin_state *state, Slapi_Entry *e,
 	}
 	n_groups = j;
 	ret = malloc((n + 1) * sizeof(char *));
-	lengths = malloc(n * sizeof(ret_lengths[0]));
-	if ((ret == NULL) || (ret_lengths == NULL)) {
+	lengths = malloc(n * sizeof((*ret_lengths)[0]));
+	if ((ret == NULL) || (lengths == NULL) || (ret_lengths == NULL)) {
 		free(ret);
 		free(lengths);
+		free(single_lengths);
 		backend_free_gathered_data(NULL, NULL,
 					   n_singles, singles,
 					   n_groups, groups, group_lengths);
@@ -298,6 +324,7 @@ backend_gather_data(struct plugin_state *state, Slapi_Entry *e,
 		lengths[k] = single_lengths[i];
 		k++;
 	}
+	free(single_lengths);
 	for (i = 0; i < n_groups; i++) {
 		if (groups[i] != NULL) {
 			for (j = 0; groups[i][j] != NULL; j++) {
@@ -321,7 +348,8 @@ backend_gather_data(struct plugin_state *state, Slapi_Entry *e,
  * determine which value should be associated with those keys, and add them to
  * the map cache. */
 void
-backend_set_entry(Slapi_Entry *e, struct backend_set_data *data)
+backend_set_entry(Slapi_PBlock *pb, Slapi_Entry *e,
+		  struct backend_set_data *data)
 {
 	char **all_keys, **all_values, *ndn, *plugin_id;
 	char **key_singles, ***key_groups, **value_singles, ***value_groups;
@@ -332,13 +360,24 @@ backend_set_entry(Slapi_Entry *e, struct backend_set_data *data)
 	plugin_id = data->common.state->plugin_desc->spd_id;
 	/* Pull out the NDN of this entry. */
 	ndn = slapi_entry_get_ndn(e);
+	if (ndn != NULL) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, plugin_id,
+				"examining input entry \"%s\"\n", ndn);
+	} else {
+		slapi_log_error(SLAPI_LOG_PLUGIN, plugin_id,
+				"input entry has no name, ignoring\n");
+		return;
+	}
 	/* Pull out the keys and value for the entry. */
-	all_keys = backend_gather_data(data->common.state, e,
+	all_keys = backend_gather_data(data->common.state, pb, e,
 				       data->common.group,
 				       data->common.set,
 				       data->key_formats,
 				       data->keys_formats,
 				       data->disallowed_chars,
+				       data->common.restrict_subtrees,
+				       data->common.ignore_subtrees,
+				       &data->common.rel_attrs,
 				       &data->common.ref_attrs,
 				       &data->common.inref_attrs,
 				       &data->common.ref_attr_list,
@@ -349,12 +388,15 @@ backend_set_entry(Slapi_Entry *e, struct backend_set_data *data)
 				       &n_key_groups,
 				       &key_groups,
 				       &key_group_lengths);
-	all_values = backend_gather_data(data->common.state, e,
+	all_values = backend_gather_data(data->common.state, pb, e,
 				         data->common.group,
 				         data->common.set,
 				         data->value_formats,
 				         data->values_formats,
 				         data->disallowed_chars,
+					 data->common.restrict_subtrees,
+					 data->common.ignore_subtrees,
+				         &data->common.rel_attrs,
 				         &data->common.ref_attrs,
 				         &data->common.inref_attrs,
 					 &data->common.ref_attr_list,
@@ -417,6 +459,10 @@ backend_map_config_filter(const char *format,
 {
 	char *ret;
 	int i, j, l;
+
+	if (format == NULL) {
+		return NULL;
+	}
 	l = 0;
 	for (i = 0; format[i] != '\0'; i++) {
 		if (format[i] == '%') {
@@ -489,7 +535,9 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 	char **use_bases, *use_entry_filter;
 	char **use_key_formats, **use_keys_formats;
 	char **use_value_formats, **use_values_formats, *use_disallowed_chars;
+	const Slapi_DN **restrict_subtrees, **ignore_subtrees;
 	int i, j;
+
 	/* Read the hard-coded defaults for a map with this name. */
 	defaults_get_map_config(map, secure, &default_filter,
 				&default_key_format, &default_keys_format,
@@ -498,6 +546,13 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 	/* Read the values from the configuration entry. */
 	bases = backend_shr_get_vattr_strlist(state, e,
 					      NIS_MAP_CONFIGURATION_BASE_ATTR);
+	restrict_subtrees = backend_shr_get_vattr_sdnlist(state, e,
+							  NIS_MAP_CONFIGURATION_RESTRICT_SUBTREES_ATTR);
+	ignore_subtrees = backend_shr_get_vattr_sdnlist(state, e,
+							NIS_MAP_CONFIGURATION_IGNORE_SUBTREES_ATTR);
+	if (ignore_subtrees == NULL) {
+		backend_shr_add_sdnlist(&ignore_subtrees, DEFAULT_IGNORE_SUBTREE);
+	}
 	entry_filter = backend_shr_get_vattr_filter(state, e,
 						    NIS_MAP_CONFIGURATION_FILTER_ATTR);
 	key_formats = backend_shr_get_vattr_strlist(state, e,
@@ -528,7 +583,7 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 			  (keys_formats ? NULL :
 			   (default_key_format ?
 			    backend_shr_dup_strlist_n((char **) &default_key_format,
-			 			      1) :
+						      1) :
 			    NULL));
 	/* Use the supplied keys-format.  If there is none, and there is also
 	 * not a supplied key-format, use the default. */
@@ -537,7 +592,7 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 			   (key_formats ? NULL :
 			    (default_keys_format ?
 			     backend_shr_dup_strlist_n((char **) &default_keys_format,
-			   			       1) :
+						       1) :
 			     NULL));
 	/* Use the supplied value-format.  If there is none, and there is also
 	 * not a supplied values-format, use the default. */
@@ -546,7 +601,7 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 			    (values_formats ? NULL :
 			     (default_value_format ?
 			      backend_shr_dup_strlist_n((char **) &default_value_format,
-			 			        1) :
+						        1) :
 			      NULL));
 	/* Use the supplied values-format.  If there is none, and there is also
 	 * not a supplied value-format, use the default. */
@@ -555,7 +610,7 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 			     (value_formats ? NULL :
 			      (default_values_format ?
 			       backend_shr_dup_strlist_n((char **) &default_values_format,
-			   			         1) :
+						         1) :
 			       NULL));
 	/* We don't supply defaults for search locations. */
 	use_bases = backend_shr_dup_strlist(bases);
@@ -579,11 +634,22 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 	ret.common.group = strdup(domain);
 	ret.common.set = strdup(map);
 	ret.common.bases = use_bases;
+	ret.common.restrict_subtrees = restrict_subtrees;
+	ret.common.ignore_subtrees = ignore_subtrees;
 	ret.common.entry_filter = use_entry_filter;
+	ret.common.rel_attrs = NULL;
+	ret.common.rel_attr_list = NULL;
+	ret.common.rel_attrs_list = NULL;
 	ret.common.ref_attrs = NULL;
 	ret.common.inref_attrs = NULL;
 	ret.common.ref_attr_list = NULL;
 	ret.common.inref_attr_list = NULL;
+	if ((getenv(NIS_PLUGIN_PROCESS_UNINTERESTING_UPDATES_ENV) == NULL) ||
+	    (atol(getenv(NIS_PLUGIN_PROCESS_UNINTERESTING_UPDATES_ENV)) == 0)) {
+		ret.common.skip_uninteresting_updates = 1;
+	} else {
+		ret.common.skip_uninteresting_updates = 0;
+	}
 	ret.disallowed_chars = use_disallowed_chars;
 	ret.key_formats = use_key_formats;
 	ret.n_key_formats = 0;
@@ -621,7 +687,7 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 					"initializing map %s in %s (3): "
 					"filter \"%s\", "
 					"key \"%s\", "
-					"value \"%s\"\n",
+					"values \"%s\"\n",
 					map, domain,
 					use_entry_filter,
 					use_key_formats[i],
@@ -658,7 +724,7 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 					"initializing map %s in %s (3): "
 					"filter \"%s\", "
 					"keys \"%s\", "
-					"value \"%s\"\n",
+					"values \"%s\"\n",
 					map, domain,
 					use_entry_filter,
 					use_keys_formats[i],
@@ -668,9 +734,16 @@ backend_set_config_read_config(struct plugin_state *state, Slapi_Entry *e,
 		ret.n_keys_formats++;
 	}
 	*pret = backend_copy_set_data(&ret);
+	if (*pret == NULL) {
+		slapi_log_error(SLAPI_LOG_PLUGIN,
+				state->plugin_desc->spd_id,
+				"out of memory initializing map %s in %s\n",
+				map, domain);
+	}
 	free(ret.common.group);
 	free(ret.common.set);
 	backend_shr_free_strlist(ret.common.bases);
+	backend_shr_free_sdnlist(ret.common.restrict_subtrees);
 	free(ret.disallowed_chars);
 	free(ret.common.entry_filter);
 	backend_shr_free_strlist(ret.key_formats);
@@ -686,19 +759,21 @@ int
 backend_set_config_entry_add_cb(Slapi_Entry *e, void *callback_data)
 {
 	char **domains, **maps;
-	int i, j, ret;
-	struct plugin_state *state;
+	int i, j;
+	struct backend_set_config_entry_add_cbdata *cbdata;
 
-	state = callback_data;
-	domains = backend_shr_get_vattr_strlist(state, e,
+	cbdata = callback_data;
+	domains = backend_shr_get_vattr_strlist(cbdata->state, e,
 						NIS_MAP_CONFIGURATION_DOMAIN_ATTR);
-	maps = backend_shr_get_vattr_strlist(state, e,
+	maps = backend_shr_get_vattr_strlist(cbdata->state, e,
 					     NIS_MAP_CONFIGURATION_MAP_ATTR);
 	for (i = 0; (domains != NULL) && (domains[i] != NULL); i++) {
 		for (j = 0; (maps != NULL) && (maps[j] != NULL); j++) {
-			ret = backend_shr_set_config_entry_add(state, e,
-							       domains[i],
-							       maps[j]);
+			backend_shr_set_config_entry_add(cbdata->state,
+							 cbdata->pb,
+							 e,
+							 domains[i],
+							 maps[j]);
 		}
 	}
 	backend_shr_free_strlist(maps);
@@ -708,12 +783,13 @@ backend_set_config_entry_add_cb(Slapi_Entry *e, void *callback_data)
 
 /* Update/initialize parameters stored in the plugin's configuration entry. */
 void
-backend_update_params(struct plugin_state *state)
+backend_update_params(Slapi_PBlock *pb, struct plugin_state *state)
 {
 	Slapi_DN *our_dn;
 	Slapi_Entry *our_entry;
 	char *tmp, **tmpv;
-	int i;
+	int i, use_be_txns;
+
 	our_dn = slapi_sdn_new_dn_byval(state->plugin_base);
 	if (our_dn == NULL) {
 		slapi_log_error(SLAPI_LOG_PLUGIN,
@@ -726,7 +802,7 @@ backend_update_params(struct plugin_state *state)
 				state->plugin_base ? "\"" : "");
 		return;
 	}
-	wrap_search_internal_get_entry(our_dn, NULL, &our_entry,
+	wrap_search_internal_get_entry(pb, our_dn, NULL, NULL, &our_entry,
 				       state->plugin_identity);
 	slapi_sdn_free(&our_dn);
 	our_dn = NULL;
@@ -774,6 +850,18 @@ backend_update_params(struct plugin_state *state)
 			    DEFAULT_TCPWRAP_NAME);
 	}
 #endif
+	use_be_txns = backend_shr_get_vattr_boolean(state, our_entry,
+						    "nsslapd-pluginbetxn",
+						    DEFAULT_PLUGIN_USE_BETXNS);
+	if (state->use_be_txns && !use_be_txns) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
+				"turning off betxn support\n");
+	}
+	if (!state->use_be_txns && use_be_txns) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
+				"turning on betxn support\n");
+	}
+	state->use_be_txns = use_be_txns;
 	slapi_entry_free(our_entry);
 }
 
@@ -822,7 +910,8 @@ backend_get_set_config_entry_cb(Slapi_Entry *e, void *callback_data)
 }
 
 void
-backend_get_set_config(struct plugin_state *state,
+backend_get_set_config(Slapi_PBlock *parent_pb,
+		       struct plugin_state *state,
 		       const char *domain, const char *map,
 		       char ***bases, char **entry_filter)
 {
@@ -860,7 +949,7 @@ backend_get_set_config(struct plugin_state *state,
 			state->plugin_desc->spd_id,
 			"searching from \"%s\" for \"%s\" for configuration\n",
 			state->plugin_base, filter);
-	pb = slapi_pblock_new();
+	pb = wrap_pblock_new(parent_pb);
 	slapi_search_internal_set_pb(pb,
 				     state->plugin_base,
 				     LDAP_SCOPE_SUBTREE,
@@ -878,6 +967,7 @@ backend_get_set_config(struct plugin_state *state,
 					  NULL,
 					  backend_get_set_config_entry_cb,
 					  NULL);
+	slapi_pblock_destroy(pb);
 	defaults_get_map_config(map, &map_secure, &default_filter,
 				NULL, NULL, NULL, NULL, NULL);
 	if (cbdata.entry_filter == NULL) {
@@ -891,7 +981,6 @@ backend_get_set_config(struct plugin_state *state,
 	free(cbdata.entry_filter);
 
 	/* Clean up. */
-	slapi_pblock_destroy(pb);
 	free(filter);
 }
 
@@ -927,12 +1016,22 @@ backend_check_empty(struct plugin_state *state,
 
 /* Scan for the list of configured domains and maps. */
 void
-backend_startup(struct plugin_state *state)
+backend_startup(Slapi_PBlock *pb, struct plugin_state *state)
 {
-	backend_shr_startup(state, NIS_MAP_CONFIGURATION_FILTER);
+	backend_shr_startup(state, pb, NIS_MAP_CONFIGURATION_FILTER);
 }
 
 /* Set up our post-op callbacks. */
+#ifdef SLAPI_NIS_SUPPORT_BE_TXNS
+int
+backend_init_betxn_postop(Slapi_PBlock *pb, struct plugin_state *state)
+{
+	slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
+			"hooking up betxn postoperation callbacks\n");
+	return backend_shr_betxn_postop_init(pb, state);
+}
+#endif
+
 int
 backend_init_postop(Slapi_PBlock *pb, struct plugin_state *state)
 {
