@@ -997,9 +997,11 @@ backend_search_entry_cb(const char *domain, const char *map, bool_t secure,
 {
 	Slapi_DN *sdn;
 	Slapi_Entry *entry;
+	Slapi_Attr *attr = NULL;
 	struct backend_search_cbdata *cbdata;
 	struct backend_entry_data *entry_data;
 	int result;
+	bool_t is_attr_exists = FALSE;
 
 	cbdata = cb_data;
 	entry_data = backend_data;
@@ -1042,7 +1044,10 @@ backend_search_entry_cb(const char *domain, const char *map, bool_t secure,
 			idview_process_overrides(cbdata, key, map, domain, entry);
 		}
 
-		if (slapi_entry_attr_exists(entry, IPA_IDVIEWS_ATTR_ANCHORUUID) == 1) {
+		/* slapi_entry_attr_exists() was introduced only in https://fedorahosted.org/389/ticket/47710 */
+		is_attr_exists = slapi_entry_attr_find(entry, IPA_IDVIEWS_ATTR_ANCHORUUID, &attr) == 0;
+
+		if (is_attr_exists == TRUE) {
 			slapi_entry_attr_delete(entry, IPA_IDVIEWS_ATTR_ANCHORUUID);
 			slapi_entry_delete_string(entry, "objectClass", "ipaOverrideTarget");
 		}
@@ -1166,6 +1171,44 @@ backend_search_set_cb(const char *group, const char *set, bool_t flag,
 	return TRUE;
 }
 
+/* Routines to search if a target DN is within any of the sets we handle */
+static bool_t
+backend_search_find_set_dn_in_group_cb(const char *group, const char *set, bool_t flag,
+					 void *backend_data, void *cb_data)
+{
+	struct backend_search_cbdata *cbdata;
+	struct backend_set_data *set_data;
+
+	cbdata = cb_data;
+	set_data = backend_data;
+
+	if (slapi_sdn_scope_test(cbdata->target_dn,
+				 set_data->container_sdn,
+				 cbdata->scope) == 1) {
+		cbdata->answer = TRUE;
+	}
+
+	if (slapi_sdn_compare(set_data->container_sdn,
+			      cbdata->target_dn) == 0) {
+		cbdata->answer = TRUE;
+	}
+
+	return TRUE;
+
+}
+
+static bool_t
+backend_search_find_set_dn_cb(const char *group, void *cb_data)
+{
+	struct backend_search_cbdata *cbdata;
+
+	cbdata = cb_data;
+	map_data_foreach_map(cbdata->state, group,
+			     backend_search_find_set_dn_in_group_cb, cb_data);
+	return TRUE;
+}
+
+/* Routines to find out the set that has the same group as requested */
 static bool_t
 backend_search_find_set_data_in_group_cb(const char *group, const char *set, bool_t flag,
 					 void *backend_data, void *cb_data)
@@ -1340,9 +1383,6 @@ backend_search_cb(Slapi_PBlock *pb)
 			"searching from \"%s\" for \"%s\" with scope %d%s\n",
 			cbdata.target, cbdata.strfilter, cbdata.scope,
 			backend_sch_scope_as_string(cbdata.scope));
-#ifdef USE_IPA_IDVIEWS
-	idview_replace_target_dn(&cbdata.target, &cbdata.idview);
-#endif
 	cbdata.target_dn = slapi_sdn_new_dn_byval(cbdata.target);
 	/* Check if there's a backend handling this search. */
 	if (!slapi_be_exist(cbdata.target_dn)) {
@@ -1351,19 +1391,47 @@ backend_search_cb(Slapi_PBlock *pb)
 				"slapi_be_exists(\"%s\") = 0, "
 				"ignoring search\n", cbdata.target);
 		slapi_sdn_free(&cbdata.target_dn);
-		if (cbdata.idview != NULL) {
-			slapi_ch_free_string(&cbdata.target);
-		}
-		slapi_ch_free_string(&cbdata.idview);
-#ifdef USE_IPA_IDVIEWS
-		idview_free_overrides(&cbdata);
-#endif
 		return 0;
 	}
 
+#ifdef USE_IPA_IDVIEWS
+	/* We may have multiple disjoint trees in the sets, search if the target matches any of them
+	 * as in general there don't have to be a single subtree (cn=compat,$SUFFIX) for all trees to easily
+	 * detect the ID view use. Unless the ID view is within the set we control, don't consider the override */
+	map_data_foreach_domain(cbdata.state, backend_search_find_set_dn_cb, &cbdata);
+	if (cbdata.answer == FALSE) {
+		idview_replace_target_dn(&cbdata.target, &cbdata.idview);
+		if (cbdata.idview != NULL) {
+			slapi_sdn_free(&cbdata.target_dn);
+			/* Perform another check, now for rewritten DN */
+			cbdata.target_dn = slapi_sdn_new_dn_byval(cbdata.target);
+			map_data_foreach_domain(cbdata.state, backend_search_find_set_dn_cb, &cbdata);
+			/* Rewritten DN might still be outside of our trees */
+			if (cbdata.answer == TRUE) {
+				slapi_log_error(SLAPI_LOG_PLUGIN, cbdata.state->plugin_desc->spd_id,
+						"Use of ID view '%s' is detected, searching from \"%s\" "
+						"for \"%s\" with scope %d%s. Filter may get overridden later.\n",
+						cbdata.idview, cbdata.target, cbdata.strfilter, cbdata.scope,
+						backend_sch_scope_as_string(cbdata.scope));
+			} else {
+				slapi_sdn_free(&cbdata.target_dn);
+				slapi_ch_free_string(&cbdata.target);
+				slapi_ch_free_string(&cbdata.idview);
+				slapi_log_error(SLAPI_LOG_PLUGIN,
+						cbdata.state->plugin_desc->spd_id,
+						"The search base didn't match any of the containers, "
+						"ignoring search\n");
+				return 0;
+			}
+		}
+	}
+	cbdata.answer = FALSE;
+#endif
+
 	/* Walk the list of groups. */
 	wrap_inc_call_level();
 #ifdef USE_IPA_IDVIEWS
+	/* Filter replacement requires increased call level as we may fetch overrides and thus come back here */
 	idview_replace_filter(&cbdata);
 #endif
 	if (map_rdlock() == 0) {
@@ -1380,10 +1448,7 @@ backend_search_cb(Slapi_PBlock *pb)
 	/* If during search of some sets we staged additional lookups, perform them. */
 	if (cbdata.staged != NULL) {
 		/* Allocate buffer to be used for getpwnam_r/getgrnam_r requests */
-		cbdata.nsswitch_buffer_len = MAX(sysconf(_SC_GETPW_R_SIZE_MAX), sysconf(_SC_GETGR_R_SIZE_MAX));
-		if (cbdata.nsswitch_buffer_len == -1) {
-			cbdata.nsswitch_buffer_len = 16384;
-		}
+		cbdata.nsswitch_buffer_len = MAX(16384, MAX(sysconf(_SC_GETPW_R_SIZE_MAX), sysconf(_SC_GETGR_R_SIZE_MAX)));
 		cbdata.nsswitch_buffer = malloc(cbdata.nsswitch_buffer_len);
 		/* Go over the list of staged requests and retrieve entries.
 		 * It is important to perform the retrieval *without* holding any locks to the map cache */
@@ -1568,7 +1633,6 @@ static void
 backend_locate(Slapi_PBlock *pb, struct backend_entry_data **data, const char **group, const char**set)
 {
 	struct backend_locate_cbdata cbdata;
-	char *idview = NULL;
 
 	slapi_pblock_get(pb, SLAPI_PLUGIN_PRIVATE, &cbdata.state);
 	if (cbdata.state->plugin_base == NULL) {
@@ -1577,22 +1641,64 @@ backend_locate(Slapi_PBlock *pb, struct backend_entry_data **data, const char **
 		return;
 	}
 	slapi_pblock_get(pb, SLAPI_TARGET_DN, &cbdata.target);
-#ifdef USE_IPA_IDVIEWS
-	idview_replace_target_dn(&cbdata.target, &idview);
-#endif
+
 	cbdata.target_dn = slapi_sdn_new_dn_byval(cbdata.target);
 	cbdata.entry_data = NULL;
 	cbdata.entry_group = NULL;
 	cbdata.entry_set = NULL;
 	map_data_foreach_map(cbdata.state, NULL, backend_locate_cb, &cbdata);
+#ifdef USE_IPA_IDVIEWS
+	/* In case nothing was found but we are operating on the ID override,
+	 * rebuild the target's RDN to use original attribute's value */
+	if (cbdata.entry_data == NULL) {
+		char *idview = NULL;
+		char *target, *original_target;
+		target = original_target = slapi_ch_strdup(cbdata.target);
+		idview_replace_target_dn(&target, &idview);
+		if (target != original_target) {
+			slapi_ch_free_string(&original_target);
+		}
+		if (idview != NULL) {
+			char *rdnstr;
+			char *val;
+			struct berval bval;
+			int res;
+			struct backend_search_cbdata scbdata;
+			Slapi_RDN *rdn = slapi_rdn_new_all_dn(target);
+			if (rdn != NULL) {
+				res = slapi_rdn_get_first(rdn, &rdnstr, &val);
+				if (res == 1) {
+					bval.bv_len = strlen(val) + 1;
+					bval.bv_val = slapi_ch_strdup(val);
+					memset(&scbdata, 0, sizeof(scbdata));
+					scbdata.idview = idview;
+					scbdata.target = target;
+					scbdata.pb = pb;
+					scbdata.state = cbdata.state;
+					scbdata.target_dn = slapi_sdn_new_dn_byval(target);
+					res = idview_replace_bval_by_override("rdn", rdnstr, &bval, &scbdata);
+					/* only accept uid overrides */
+					if (res == 1) {
+						slapi_rdn_remove_index(rdn, 1);
+						slapi_rdn_add(rdn, "uid", bval.bv_val);
+						slapi_sdn_free(&cbdata.target_dn);
+						cbdata.target_dn = slapi_sdn_set_rdn(scbdata.target_dn, rdn);
+						map_data_foreach_map(cbdata.state, NULL, backend_locate_cb, &cbdata);
+					}
+					slapi_ber_bvdone(&bval);
+					slapi_rdn_free(&rdn);
+					idview_free_overrides(&scbdata);
+				}
+			}
+		}
+		slapi_ch_free_string(&target);
+		slapi_ch_free_string(&idview);
+	}
+#endif
 	*data = cbdata.entry_data;
 	*group = cbdata.entry_group;
 	*set = cbdata.entry_set;
 	slapi_sdn_free(&cbdata.target_dn);
-	if (idview != NULL) {
-		slapi_ch_free_string(&cbdata.target);
-	}
-	slapi_ch_free_string(&idview);
 }
 
 /* Check if the target DN is part of this group's tree.  If it is, return an
