@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat, Inc.
+ * Copyright 2008,2012 Red Hat, Inc.
  *
  * This Program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ struct plugin_state {
 	char *plugin_base;
 	Slapi_ComponentId *plugin_identity;
 	Slapi_PluginDesc *plugin_desc;
+	unsigned int use_be_txns: 1;
 
 	/* NIS-specific data. */
 	struct wrapped_thread *tid;
@@ -43,6 +44,9 @@ struct plugin_state {
 	struct {
 		int fd, port, pf, type;
 	} listener[4];
+	/* Schema compat-specific data. */
+	struct wrapped_rwlock *pam_lock;
+	void *nss_context;
 };
 
 #endif
